function y = TreeOptions(S,Strike,r,dt,pu,pm,pd,N,EuroAmer,PutCall)

% Calculates the prices of American and European calls and puts from an
% implied volatility tree.
% Inputs are
% S = Stock price (vector size 2*N+1)
% Strike = Strike price (scalar).
% r  = Risk free rate (scalar).
% dt = Time increment from the implied tree (scalar).
% pu = "up"     probabilities (matrix 2*N+1 by N+1)
% pm = "middle" probabilities (matrix 2*N+1 by N+1)
% pd = "down"   probabilities (matrix 2*N+1 by N+1)
% N  = Number of time steps.
% EuroAmer = 'E'uropean or 'A'merican
% PutCall  = 'P'ut or 'C'all.

V = zeros(2*N+1,N+1);

if strcmp(PutCall,'C')
	V(:,end) = max(S - Strike,0);
elseif strcmp(PutCall,'P')
	V(:,end) = max(Strike - S,0);
end
	
for j=N:-1:1
	for i=N-j+2:N+j
		switch EuroAmer
			case 'A'
				if strcmp(PutCall,'C')
					V(i,j) = max(exp(-r*dt)*(pu(i,j)*V(i-1,j+1) + pm(i,j)*V(i,j+1) + pd(i,j)*V(i+1,j+1)), S(i) - Strike);
				elseif strcmp(PutCall,'P')
					V(i,j) = max(exp(-r*dt)*(pu(i,j)*V(i-1,j+1) + pm(i,j)*V(i,j+1) + pd(i,j)*V(i+1,j+1)), Strike - S(i));
				end
			case 'E'
					V(i,j) = exp(-r*dt)*(pu(i,j)*V(i-1,j+1) + pm(i,j)*V(i,j+1) + pd(i,j)*V(i+1,j+1));
		end
	end
end

y = V(N+1,1);
	
