function [pu pm pd] = buildProbs(N,Q,infl,S);

% Build the "up", "middle" and "down" probabilities for the implied
% trinomial tree.
% Inputs are
%   N = number of time steps (scalar).
%   Q = Arrow-Debreu prices (matrix 2N+1 by N+1).
%   infl = "inflation" factor.
%   S = Stock prices (vector 2N+1)  

pu = zeros(2*N+1,N+1);     % Up probabilities.
pm = zeros(2*N+1,N+1);     % Middle probabilities.
pd = zeros(2*N+1,N+1);     % Down probabilities.
st = zeros(2*N+1,N+1);     % Stability check.

% Upper and middle nodes.
for j=N:-1:1;
	for i=N-j+2:N+1
		if i==N-j+2
			pu(i,j) =  infl*Q(i-1,j+1)/Q(i,j); 
			pm(i,j) = (infl*S(i) - S(i+1) - pu(i,j)*(S(i-1)-S(i+1))) / (S(i) - S(i+1));
			pd(i,j) = 1 - pu(i,j) - pm(i,j);			
		elseif i==N-j+3
			pu(i,j) = (infl*Q(i-1,j+1) - pm(i-1,j)*Q(i-1,j)) / Q(i,j);
			pm(i,j) = (infl*S(i) - S(i+1) - pu(i,j)*(S(i-1)-S(i+1))) / (S(i) - S(i+1));
			pd(i,j) = 1 - pu(i,j) - pm(i,j);
		else
 			pu(i,j) = (infl*Q(i-1,j+1) - pd(i-2,j)*Q(i-2,j) - pm(i-1,j)*Q(i-1,j)) / Q(i,j);
 			pm(i,j) = (infl*S(i) - S(i+1) - pu(i,j)*(S(i-1)-S(i+1))) / (S(i) - S(i+1));
			pd(i,j) = 1 - pu(i,j) - pm(i,j);

		end
		st(i,j) = sqrt(3*(pu(i,j) + pd(i,j) - pu(i,j)^2 + 2*pu(i,j)*pd(i,j) - pd(i,j)^2));
	end
end

% Lower nodes.
for j=N:-1:2
	for i=N+j:-1:N+2
		if i==N+j
			pd(i,j) = infl*Q(i+1,j+1)/Q(i,j);
			pm(i,j) = (infl*S(i) - S(i-1) - pd(i,j)*(S(i+1) - S(i-1))) / (S(i) - S(i-1));
			pu(i,j) = 1 - pd(i,j) - pm(i,j);
		elseif i==N+j-1
			pd(i,j) = (infl*Q(i+1,j+1) - pm(i+1,j)*Q(i+1,j)) / Q(i,j);
			pm(i,j) = (infl*S(i) - S(i-1) - pd(i,j)*(S(i+1) - S(i-1))) / (S(i) - S(i-1));
			pu(i,j) = 1 - pd(i,j) - pm(i,j);
		else
			pd(i,j) = (infl*Q(i+1,j+1) - pu(i+2,j)*Q(i+2,j) - pm(i+1,j)*Q(i+1,j)) / Q(i,j);
			pm(i,j) = (infl*S(i) - S(i-1) - pd(i,j)*(S(i+1) - S(i-1))) / (S(i) - S(i-1));
			pu(i,j) = 1 - pd(i,j) - pm(i,j);
		end
		st(i,j) = sqrt(3*(pu(i,j) + pd(i,j) - pu(i,j)^2 + 2*pu(i,j)*pd(i,j) - pd(i,j)^2));
	end
end
